<?php
/**
 * Minimee: minimize & combine your CSS and JS files.
 * @version 1.0.0 (2010-10-06)
 * @author John D Wells <http://johndwells.com>
 * @license http://www.opensource.org/licenses/bsd-license.php BSD license, as per the original Minify_CSS class
 **/

if ( ! defined('BASEPATH')) exit('No direct script access allowed');

class Minimee_ext {

	var $name		= 'Minimee';
	var $version 		= '1.0.0';
	var $description	= 'Minimee: minimize & combine your CSS and JS files.';
	var $settings_exist	= 'y';
	var $docs_url		= '';

	var $settings        = array();
	
	var $EE;

	/**
	 * PHP4 Constructor
	 *
	 * @param 	mixed	Settings array or empty string if none exist.
	 */
	function Minimee_ext($settings = array())
	{
		$this->__construct();
	}
	// END

	
	/**
	 * PHP5 Constructor
	 *
	 * @param 	mixed	Settings array or empty string if none exist.
	 */
	function __construct($settings = array())
	{
		$this->EE =& get_instance();
	}
	// END
	
	
	/**
	 * Used by plugin, retrieves settings from database
	 *
	 * @return void
	 */
	function get_settings()
	{

		// first check for settings in session cache	
		if( isset($this->EE->session->cache['minimee']['settings']) ) {
			$this->settings = $this->EE->session->cache['minimee']['settings'];
			return $this->settings;
		}

		// if we are here, then pull from DB
		$this->EE->db
					->select('settings')
					->from('extensions')
					->where(array('enabled' => 'y', 'class' => __CLASS__ ))
					->limit(1);
		$query = $this->EE->db->get();
		
		if ($query->num_rows() > 0 ) {
			$this->settings = unserialize($query->row()->settings);
		} else {
			$this->settings = array();
		}
		
		// now set to session for next time
		if(! array_key_exists('minimee', $this->EE->session->cache)) {
			$this->EE->session->cache['minimee'] = array();
		}
		$this->EE->session->cache['minimee']['settings'] = $this->settings;

	}
	// END

	
	/**
	 * Activate Extension
	 * @return void
	 */
	function activate_extension()
	{
		$this->settings = array(
			'cache_url'		=> '',
			'cache_path'	=> '',
			'disable'		=> 'false',
			'strict'		=> 'false'
		);
		
		
		$data = array(
			'class'		=> __CLASS__,
			'settings'	=> serialize($this->settings),
			'priority'	=> 10,
			'version'	=> $this->version,
			'enabled'	=> 'y'
		);
		
		$this->EE->db->insert('extensions', $data);
	}
	// END


	/**
	 * Update Extension
	 *
	 * @param 	string	String value of current version
	 * @return 	mixed	void on update / false if none
	 */
	function update_extension($current = '')
	{
		if ($current == '' OR $current == $this->version)
		{
			return FALSE;
		}
		
		if ($current < '1.0.0')
		{
			// Update to version 1.0.0
		}
		
		$this->EE->db->where('class', __CLASS__);
		$this->EE->db->update(
					'extensions', 
					array('version' => $this->version)
		);
	}
	// END


	/**
	 * Disable Extension
	 *
	 * @return void
	 */
	function disable_extension()
	{
		$this->EE->db->where('class', __CLASS__);
		$this->EE->db->delete('extensions');
	}
	// END


	/**
	 * Minimal function required to use DB settings
	 *
	 * @return void
	 */
	function settings()
	{
	    $settings = array();
	    
		$disable = (array_key_exists('minimee_disable', $this->EE->config->_global_vars))
							? strtolower($this->EE->config->_global_vars['minimee_disable'])
							: FALSE;
	    $disable = ($disable == 'yes' || $disable == 'true' || $disable == '1') ? 'true' : 'false'; // default = FALSE


		$strict = (array_key_exists('minimee_strict', $this->EE->config->_global_vars))
							? strtolower($this->EE->config->_global_vars['minimee_strict'])
							: FALSE;
	    $strict = ($strict === TRUE || $strict == 'yes' || $strict == 'true' || $strict == '1') ? 'true' : 'false'; // default = FALSE


		$cache_path = (array_key_exists('minimee_cache_path', $this->EE->config->_global_vars))
							? $this->EE->config->_global_vars['minimee_cache_path']
							: '';

		$cache_url = (array_key_exists('minimee_cache_url', $this->EE->config->_global_vars))
							? $this->EE->config->_global_vars['minimee_cache_url']
							: '';
	    
	    $settings['cache_path']    = $cache_path;
	    $settings['cache_url']    = $cache_url;
	    $settings['disable']    = array('r', array('false' => 'No', 'true' => 'Yes'), $disable);
	    $settings['strict']    = array('r', array('false' => 'No', 'true' => 'Yes'), $strict);
	    
	    return $settings;
	}
	// END
	
}
// END CLASS

	
/* End of file ext.minimee.php */ 
/* Location: ./system/expressionengine/third_party/minimee/ext.minimee.php */