<?php
/**
 * Minimee: minimize & combine your CSS and JS files.
 * @version 1.0.0 (2010-10-06)
 * @author John D Wells <http://johndwells.com>
 * @license http://www.opensource.org/licenses/bsd-license.php BSD license, as per the original Minify_CSS class
 **/

if ( ! defined('BASEPATH')) exit('No direct script access allowed');

$plugin_info = array(
	'pi_name' => 'Minimee',
	'pi_version' =>'1.0.0',
	'pi_author' =>'John D Wells',
	'pi_author_url' => 'http://johndwells.com/software',
	'pi_description' => 'Minimee: minimize & combine your CSS and JS files.',
	'pi_usage' => Minimee::usage()
);

class Minimee {

	var $cache_path;
	var $cache_url;
	var $media;
	var $minify;
	var $combine;
	var $disable;
	var $queue;
	var $strict;

	var $files = array();
    var $EE;
    var $ext;

	/**
	 * PHP4 constructor
	 */
	function Minimee() {
		$this->__construct();
	}
	// END

	
	/**
	 * PHP5 constructor
	 */
	function __construct() {
		$this->EE =& get_instance();

		// Get extension settings from database
		if(! class_exists('Minimee_ext')) {
			require_once(PATH_THIRD . 'minimee/ext.minimee.php');
		}
		$this->ext = new Minimee_ext();
		$this->ext->get_settings();

		// now get all settings - first from DB, then from config file

		/**** DISABLE ***/
		$this->disable = (array_key_exists('disable', $this->ext->settings))
							? strtolower($this->ext->settings['disable'])
							: FALSE;
		$this->disable = (array_key_exists('minimee_disable', $this->EE->config->_global_vars))
							? strtolower($this->EE->config->_global_vars['minimee_disable'])
							: $this->disable;
	    $this->disable = ($this->disable == 'yes' || $this->disable == 'true' || $this->disable == '1') ? TRUE : FALSE; // default = FALSE


		/**** STRICT ***/
		$this->strict = (array_key_exists('strict', $this->ext->settings))
							? strtolower($this->ext->settings['strict'])
							: FALSE;
		$this->strict = (array_key_exists('minimee_strict', $this->EE->config->_global_vars))
							? strtolower($this->EE->config->_global_vars['minimee_strict'])
							: $this->strict;
	    $this->strict = ($this->strict === TRUE || $this->strict == 'yes' || $this->strict == 'true' || $this->strict == '1') ? TRUE : FALSE; // default = FALSE


		/**** CACHE_PATH ***/
		$this->cache_path = (array_key_exists('cache_path', $this->ext->settings))
							? strtolower($this->ext->settings['cache_path'])
							: FALSE;
		$this->cache_path = (array_key_exists('minimee_cache_path', $this->EE->config->_global_vars))
							? rtrim($this->EE->config->_global_vars['minimee_cache_path'], '/')
							: $this->cache_path;


		/**** CACHE_URL ***/
		$this->cache_url = (array_key_exists('cache_url', $this->ext->settings))
							? strtolower($this->ext->settings['cache_url'])
							: FALSE;
		$this->cache_url = (array_key_exists('minimee_cache_url', $this->EE->config->_global_vars))
							? rtrim($this->EE->config->_global_vars['minimee_cache_url'], '/')
							: $this->cache_url;
		
		
		// Final flightcheck: determine if we can continue or disable permanently
		switch('flightcheck') :
			case(! $this->cache_path) :
			case(! $this->cache_url) :
			case(! file_exists($this->cache_path)) :
				$this->disable = TRUE;
				log_message('error', 'Minimee not configured, or configured incorrectly.');
			break;
			
			default :
				log_message('debug', 'Minimee plugin initialized.');
		endswitch;
	}

	/**
	 * Plugin function: exp:minimee:css
	 * 
	 * @return mixed string or empty
	 */
	function css()
	{
		return $this->_run('css');
	}
	// END


	/**
	 * Plugin function: exp:minimee:js
	 * 
	 * @return mixed string or empty
	 */
	function js()
	{
		return $this->_run('js');
	}
	// END


	/**
	 * Plugin function: exp:minimee:display
	 * 
	 * @return mixed string or empty
	 */
	function display()
	{

		// see which to display
	    $js = strtolower($this->EE->TMPL->fetch_param('js'));
	    $css = strtolower($this->EE->TMPL->fetch_param('css'));
	    $out = '';

	    if($js) {
	    	$out .= $this->_display('js');
	    }

	    if($css) {
	    	$out .= $this->_display('css');
	    }
		
		return $out;
	}
	// END


	/**
	 * Courtesy of Experience Internet, behind the brilliant SL Combinator
	 * 
	 * @param string tagdata
	 * @param string either css or js
	 * @return void
	 */
	function _fetch_files($haystack, $type)
	{
		$ret = FALSE;
		
		switch(strtolower($type))
		{
			case 'css':
				$pat = "/<link{1}.*?href=['|\"']{1}(.*?)['|\"]{1}/i";
				break;
				
			case 'js':
				$pat = "/<script{1}.*?src=['|\"]{1}(.*?)['|\"]{1}/i";
				break;
				
			default:
				return FALSE;
				break;
		}
		
		if (preg_match_all($pat, $haystack, $matches, PREG_PATTERN_ORDER))
			$this->files = $matches[1];

		// if in strict mode, be sure every file exists
		$this->_check_strict();
	}
	// END
	

	/**
	 * Fetch parameters from $this->EE->TMPL
	 * 
	 * @param string either css or js
	 * @return mixed string or empty
	 */
	function _fetch_params($type = '')
	{

		// if we pass a type in, then substitute that param value for queue
		// this only happens when Minimee::display() is called,
		// and a queued list of files needs to be output
		// type will either be 'js' or 'css'
		if($type) {
		    $this->queue = strtolower($this->EE->TMPL->fetch_param($type));
		    $this->queue = ($this->queue == '' || $this->queue == 'false' || $this->queue == 'no' || $this->queue == '0') ? FALSE : $this->queue; // default = FALSE
		} else {
		    $this->queue = strtolower($this->EE->TMPL->fetch_param('queue'));
		    $this->queue = ($this->queue == '' || $this->queue == 'false' || $this->queue == 'no' || $this->queue == '0') ? FALSE : $this->queue; // default = FALSE
		}
	
	    $this->media = strtolower($this->EE->TMPL->fetch_param('media'));
	    $this->media = ($this->media != '') ? $this->media : 'screen'; // default = 'screen'

	    $this->combine = strtolower($this->EE->TMPL->fetch_param('combine'));
	    $this->combine = ($this->combine == 'false' || $this->combine == 'no' || $this->combine == '0') ? FALSE : TRUE; // default = TRUE

	    $this->minify = strtolower($this->EE->TMPL->fetch_param('minify'));
	    $this->minify = ($this->minify == 'false' || $this->minify == 'no' || $this->minify == '0') ? FALSE : TRUE; // default = TRUE
	    
	    $this->disable = strtolower($this->EE->TMPL->fetch_param('disable'));
	    $this->disable = ($this->disable == 'yes' || $this->disable == 'true' || $this->disable == '1') ? TRUE : FALSE; // default = FALSE
	}
	// END


	/**
	 * Called by Minimee:css and Minimee:js, performs basic run command
	 * 
	 * @param string either 'js' or 'css'
	 * @return mixed string or empty
	 */
	function _run($type)
	{
		// Globally disabled?
		if ($this->disable) return $this->EE->TMPL->tagdata;
		
		$this->_fetch_params();

		// Disabled?
		if ($this->disable) return $this->EE->TMPL->tagdata;

		$this->_fetch_files($this->EE->TMPL->tagdata, $type);

		// One final check if disabled - possibly caused by strict
		if ($this->disable) return $this->EE->TMPL->tagdata;

		// No files found?
		if( !is_array($this->files) || count($this->files) == 0 ) return $this->EE->TMPL->tagdata;

		// If for some reason we do not want to minify or combine, then why are we here?
		if(!$this->combine && !$this->minify) return $this->EE->TMPL->tagdata;
		
		// Are we queueing for later? If so, just save in session
		if($this->queue) return $this->_set_queue($type);
		
		return $this->_out($type);
	}
	
	/** 
	 * Adds the files to be queued into session
	 * 
	 * @param string either 'js' or 'css'
	 * @return void
	 */
	function _set_queue($type)
	{
		if(! array_key_exists('minimee', $this->EE->session->cache))
			$this->EE->session->cache['minimee'] = array();
		
		if(! array_key_exists($type, $this->EE->session->cache['minimee']))
			$this->EE->session->cache['minimee'][$type] = array();
		
		if(! array_key_exists($this->queue, $this->EE->session->cache['minimee'][$type]))
			$this->EE->session->cache['minimee'][$type][$this->queue] = array();

		foreach($this->files as $file) {
			if(! in_array($file, $this->EE->session->cache['minimee'][$type][$this->queue]))
				$this->EE->session->cache['minimee'][$type][$this->queue][] = $file; 
		}
	}
	// END
	

	/**
	 * Retrieve files from cache
	 *
	 * @param string either 'js' or 'css'
	 * @return void
	 */	
	function _fetch_queue($type)
	{
		if(! array_key_exists('minimee', $this->EE->session->cache)) return;

		if(! array_key_exists($type, $this->EE->session->cache['minimee'])) return;

		if(! array_key_exists($this->queue, $this->EE->session->cache['minimee'][$type])) return;
		
		// clear queue just in case
		$this->files = array();

		foreach($this->EE->session->cache['minimee'][$type][$this->queue] as $file) {
			$this->files[] = $file;
		}

		// if in strict mode, be sure every file exists
		$this->_check_strict();
	}
	// END
	
	
	/**
	 * If in strict mode, verify first that all files exist
	 *
	 * @return void
	 */	
	function _check_strict()
	{
	
		// if in strict mode, be sure every file exists
		if($this->strict) :
			foreach($this->files as $file ) :

				if($this->_isURL($file)) {
					// do not test for remote files
				} elseif ( ! realpath( FCPATH . '/' . $file ) ) {
					$this->disable = true;
					return;
				}

			endforeach;
		endif;
	}
	// END
	
	
	/**
	 * Processes and displays queue
	 *
	 * @param string either 'js' or 'css'
	 * @return string The final output from Minimee::out()
	 */	
	function _display($type)
	{
		// Globally disabled?
		if ($this->disable) return;
		
		$this->_fetch_params($type);

		// Locally Disabled?
		if ($this->disable) return;

		// Was a queue passed?
		if (! $this->queue) return;

		$this->_fetch_queue($type);

		// Last check if Disabled, cause by strict mode
		if ($this->disable) return;
		
		// No files found?
		if( !is_array($this->files) || count($this->files) == 0 ) return;

		return $this->_out($type);
	}
	// END
	
	
	/**
	 * Performs heavy lifting of plugin, processing output and returning final tags
	 *
	 * @param string either 'js' or 'css'
	 * @return string The final tag to be returned to template
	 */	
	function _out($type)
	{

		// If for some reason we do not want to minify or combine, then why are we here?
		if(!$this->combine && !$this->minify) return $this->EE->TMPL->tagdata;
		
		// our return variable	
		$out = '';

		// if we are not combining, then minify each file
		if (!$this->combine) {

			foreach($this->files as $file) :

				if( $this->_isURL($file)) {

					$filename = '0000000000' . md5($file) . '.' . $type;

					if( !file_exists($this->cache_path . '/' . $filename) ) {
						$file_data = $this->_minify($type, $file) . "\n";
						$this->_cache($filename, $file_data);
					}
	
					$out .= $this->_tag($type,  $filename, TRUE, $this->media);

			
				} elseif( realpath( FCPATH . '/' . $file ) ) {

					$lastmodified = 0;
					$lastmodified = filemtime( realpath( FCPATH . '/' . $file ) );
					$lastmodified = ($lastmodified == 0) ? '0000000000' : $lastmodified;
					$filename = $lastmodified . md5($file) . '.' . $type;
			
					if( !file_exists($this->cache_path . '/' . $filename) ) {
						$file_data = $this->_minify($type, $file) . "\n";
						$this->_cache($filename, $file_data);
					}
	
					$out .= $this->_tag($type,  $filename, TRUE, $this->media);
				}

			endforeach;

		// combine files, and possibly minify
		} else {

			$lastmodified = 0;
			$cache_name = '';
			
			foreach($this->files as $file) :
				if( $this->_isURL($file)) {
					$lastmodified = '0000000000';
					$cache_name .= $file;
				} elseif( realpath( FCPATH . '/' . $file ) ) {
					$lastmodified = max($lastmodified, filemtime( realpath( FCPATH . '/' . $file ) ) );
					$cache_name .= $file;
				}
			endforeach;
			$lastmodified = ($lastmodified == 0) ? '0000000000' : $lastmodified;
			$filename = $lastmodified . md5($cache_name) . '.' . $type;
	
			if( !file_exists($this->cache_path . '/' . $filename) ) $this->_combine($type, $this->files, $filename, $this->minify);

			$out = $this->_tag($type, $filename, TRUE, $this->media);
		}
		
		return $out;

	}
	//END
	
	
	/** 
	* Internal function for compressing/combining scripts
	* @param	String flag the asset type: css|js
	* @param	array of file references to be combined. Should contain arrays, as included in primary asset arrays: ('dev'=>$dev, 'prod'=>$prod, 'minify'=>TRUE||FALSE)
	* @param	String of the filename of the file-to-be
	* @return   None
	*/
	function _combine($flag, $files, $filename, $minify)
	{

		$file_data = '';
		foreach($files as $file):

			if( $minify ) :
				
				$file_data .=  $this->_minify( $flag, $file ) . "\n";
				
			else :

				switch($flag) :

					case 'js' :

						if( $this->_isURL($file)) :
							$file_data .=  file_get_contents( $file ) . "\n";
						else :
							$file = realpath( FCPATH . '/' . $file );

							// need a path value
							if(! $file) continue;
							
							// file must exist
							if(! file_exists($file)) continue;
			
							$file_data .=  file_get_contents( $file ) ."\n";
						endif;

					break;
					case 'css' :

						if( $this->_isURL($file)) :
							$file_data .=  file_get_contents( $file ) . "\n";
						else :
							$file = realpath( FCPATH . '/' . $file );
							$file_data .=  file_get_contents( $file ) ."\n";
						endif;
					break;
				endswitch;
			endif;
		
		endforeach;
		
		$this->_cache( $filename, $file_data );

	}
	// END


	/** 
	* Internal function for minifying assets
	* @param	String flag the asset type: css|js
	* @param	String of the path to the file whose contents should be minified
	* @return   String minified contents of file
	*/
	function _minify($flag, $file)
	{

		$path = ( $this->_isURL($file) ) ? $file : realpath(FCPATH . '/' . $file);
		
		// need a path value
		if(! $path) return;
		
		// file must exist if not a URL
		if( ! $this->_isURL($file) && ! file_exists($path) ) return;

		
		switch($flag){
			
			case 'js':

				require_once('libraries/jsmin.php');
				$this->jsmin = new jsmin();

				$contents = file_get_contents( $path );
				return $this->jsmin->minify($contents);
			
			break;
			
			
			case 'css':
				require_once('libraries/cssmin.php');
				$this->cssmin = new cssmin();

				$rel = ( $this->_isURL($file) ) ? $file : dirname(rtrim($this->EE->config->config['base_url'], '/') . '/' . $file) . '/';
				$this->cssmin->config(array('relativePath'=>$rel));
				
				$contents = file_get_contents( $path );
				return $this->cssmin->minify($contents);
			
			break;
		}
	
	}
	// END
	
	
	/** 
	* Internal function for writing cache files
	* @param	String of filename of the new file
	* @param	String of contents of the new file
	* @return   boolean	Returns true on successful cache, false on failure
	*/
	function _cache($filename, $file_data)
	{

		$filepath = $this->cache_path . '/' . $filename;
		$success = file_put_contents( $filepath, $file_data );
		
		if($success) : 
			log_message('debug', 'MiniMee: Cache file '.$filename.' was written to '.$this->cache_path);
			return TRUE;
		else : 
			log_message('error', 'Minimee: There was an error writing cache file '.$filename.' to '.$this->cache_path);
			return FALSE;
		endif;
	}
	// END
	
	
	/** 
	* Internal function for making tag strings
	* @param	String flag for type: css|js
	* @param	String of reference of file. 
	* @param	Boolean flag for cache dir.  Defaults to FALSE.
	* @param	String Media type for the tag.  Only applies to CSS links. defaults to 'screen'
	* @return	String containing an HTML tag reference to given reference
	*/
	function _tag($flag, $ref, $cache = FALSE, $media = 'screen')
	{

		switch($flag){
		
			case 'css':
				
				$dir = ( $this->_isURL($ref) ) ? '' : ( ($cache) ? $this->cache_url : rtrim($this->EE->config->config['base_url'], '/') );
				return '<link type="text/css" rel="stylesheet" href="'.$dir.'/'.$ref.'" media="'.$media.'" />'."\r\n";
			
			break;

			case 'js':
				
				$dir = ( $this->_isURL($ref) ) ? '' : ( ($cache) ? $this->cache_url : rtrim($this->EE->config->config['base_url'], '/') );
				
				return '<script type="text/javascript" src="'.$dir.'/'.$ref.'" charset="'.$this->EE->config->item('charset').'"></script>'."\r\n";
			
			break;
		
		}
	
	}
	// END

	
	/**
	* isURL
	* Allows for port, path and query string validations.  This should probably be moved into
	* a helper file, but I hate to add a whole new file for one little 2-line function.
	* [Direct inclusion from CodeIgniter Carabiner library]
	* @author Tony Dewan <tony@tonydewan.com>
	* @param	string	string containing url user input
	* @return   boolean	Returns TRUE/FALSE
	*/
	function _isURL($url)
	{
		$pattern = '@(https?://([-\w\.]+)+(:\d+)?(/([\w/_\.]*(\?\S+)?)?)?)@';
		return preg_match($pattern, $url);
	}
	// END


	/**
	 * Display usage notes in EE control panel
	 *
	 * @return string Usage notes
	 */	
	function usage() {
		ob_start(); 
?>

Note: Please refer to the readme.txt file for installation & configuration instructions.

=====================================================
Basic Usage
=====================================================

If you are familiar with SL Combinator (as I'm sure you are!), then basic usage is identical: simply wrap your normal <link> and <script> HTML tags with Minimee's tag pair, and it will parse your tags and do it's magic.

CSS:
{exp:minimee:css media="screen"}
	<link type="text/css" rel="stylesheet" href="/css/screen.css" media="screen" />
{/exp:minimee:css}

JS:
{exp:minimee:js}
	<script type="text/javascript" src="scripts/jquery.easing.1.3.js"></script>
{/exp:minimee:js}


=====================================================
Parameters
=====================================================

media="screen" (CSS only)
- sets the media attribute for the final output of the <link> tag

disable="false"
- set to "TRUE" if you wish to temporarily disable this usage of plugin

minify="true"
- tells the plugin whether to minify files when caching; if set to FALSE, it will not run files through minify engine

combine="true"
- tells plugin whether to combine files when caching; if set to FALSE, it will cache each file separately

queue="name"
- Allows you to create a queue of assets to be output at a later stage in code.
- The string passed to the queue parameter can later be used in the exp:minimee:display single tags.

* Note: if minify & combine are both set to false, it performs no caching, effectively disabling plugin


=====================================================
Advanced Usage - Queueing files for later output
=====================================================

The "queue" function allows you to collect a list of files to be compressed, minified and cached at a later point in the template parsing.  This feature relies on a clear understanding of EE's parsing order.  In particular, keep this in mind:

* EE's parsing engine processes sub-templates (e.g. embeds) LAST, each in turn
  Read more of EE's parsing order at: http://expressionengine.com/wiki/Parse_Order/

The "queue" function therefore is really only useful when your setup involves the following:

- one or more embeds, as well as the original template file, includes queue calls to minimee
- the exp:minimee:display tag is called within an EMBED that follows AFTER all other embeds that might add to the queue

Practically speaking, this feature was built specifically with JS in mind - we know it's best practice to include JS at the end of our source, but with normal embedding this is sometimes not as simple to achieve.

So for example, you might have a header embed that queues a few JS & CSS files:

{exp:minimee:js queue="main"}
	<script type="text/javascript" src="scripts/jquery.1.4.2.js"></script>
	<script type="text/javascript" src="scripts/jquery.easing.1.3.js"></script>
{/exp:minimee:js}

{exp:minimee:css queue="screen"}
	<link type="text/css" rel="stylesheet" href="css/screen.css" media="screen" />
{/exp:minimee:css}

Later on in another embed, you may need to call up other files:

{exp:minimee:js queue="main"}
	<script type="text/javascript" src="scripts/jquery.cycle.js"></script>
{/exp:minimee:js queue="main"}

{exp:minimee:css queue="screen"}
	<link type="text/css" rel="stylesheet" href="css/uniform.css" media="screen" />
{/exp:minimee:css}


Then you have a footer embed that finally outputs your files. You can either combine the output, or call one queue at a time:

{exp:minimee:display js="main" css="screen"}

- or -

{exp:minimee:display js="main"}
{exp:minimee:display css="screen"}




		
<?php
		$buffer = ob_get_contents();
		ob_end_clean(); 
		return $buffer;
	}
		}
	
/* End of file pi.minimee.php */ 
/* Location: ./system/expressionengine/third_party/minimee/pi.minimee.php */
